// Setup translations
import i18next from "i18next";
import i18nextHttpBackend from "i18next-http-backend";
import i18nextBrowserLanguageDetector from "i18next-browser-languagedetector";

export const languages = [
  "de",
  "en",
  "es",
  "fr",
  "it",
  "ja",
  "nl",
  "pt-BR",
  "ru",
  "sv",
  "zh-CN",
  "zh-TW",
  "zh-HK",
  "zh",
];

class Translator {
  constructor() {
    this.needsUpdateContent = false; // flag to know if we should do an update later
    let initialized = false; // prevent languageChanged events from firing before init is done
    this.listeners = [];
    i18next.on("languageChanged", () => {
      if (initialized) {
        this.updateContent(i18next.t);
      }
    });
    i18next
      .use(i18nextHttpBackend)
      .use(i18nextBrowserLanguageDetector)
      .init({
        fallbackLng: "en",
        supportedLngs: languages,
        debug: false,
        ns: ["common"],
        defaultNS: "common",
        backend: {
          loadPath: "/popup/i18n/{{ns}}-{{lng}}.json",
          crossDomain: false,
        },
      })
      .then((t) => {
        initialized = true;
        this.updateContent(t);
      })
      .catch((err) => console.log("trouble loading i18n", err));
  }

  addListener(listener) {
    this.listeners.push(listener);
  }

  // -----------------------------------------------------------------------
  // i18n
  // -----------------------------------------------------------------------
  updateContent(t) {
    this.listeners.forEach((listener) => {
      listener(t);
    });
    this.needsUpdateContent = false;
    this.translateId(t, "allow");
    this.translateId(t, "clear");
    this.translateId(t, "allowing");
    this.translateClass(t, "resume");
    this.translateClass(t, "pause");
    this.translateClass(t, "allow");
    // try again in a little while - sometimes we can be just a little too fast
    if (this.needsUpdateContent) {
      setTimeout(() => this.updateContent(t), 100);
    }
  }

  /**
   * changeLanguage updates and forces the language to match the given
   * language `lng`. The language change will be sent to all listeners
   * when they are ready.
   *
   * @param lng {string} language to select
   */
  changeLanguage(lng) {
    i18next.changeLanguage(lng).then((t) => {
      this.updateContent(t);
    });
  }

  /**
   * Translate using the given translator assuming the id and translation key
   * is the same.
   *
   * @param t {function} translation function
   * @param id {string} id if the element to set innerText
   */
  translateId(t, id) {
    const el = document.getElementById(id);
    if (el) {
      el.innerText = t(id, {});
    } else {
      this.needsUpdateContent = true;
    }
  }

  /**
   * Translate using the given translator all elements wih the class name
   * `<name>-text` and translation key name.
   *
   * @param t {function} translation function
   * @param name {string} name of the element class and translation key
   */
  translateClass(t, name) {
    const elements = document.getElementsByClassName(name + "-text");
    if (elements.length === 0) {
      this.needsUpdateContent = true;
      return;
    }
    const text = t(name, {});
    for (let i = 0; i < elements.length; i++) {
      elements[i].innerText = text;
    }
  }
}

export const translator = new Translator();
